<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2004-2010 The Cacti Group                                 |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

include("./include/auth.php");
include_once(CACTI_BASE_PATH . "/lib/utility.php");

define("MAX_DISPLAY_PAGES", 21);

$device_actions = array(
	ACTION_NONE => __("None"),
	"1" => __("Delete"),
	"2" => __("Duplicate")
	);

/* set default action */
if (!isset($_REQUEST["action"])) { $_REQUEST["action"] = ""; }


switch (get_request_var_request("action")) {
	case 'save':
		device_template_form_save();

		break;
	case 'save_gt':
		device_template_form_save_gt();

		break;
	case 'save_dq':
		device_template_form_save_dq();

		break;
	case 'actions':
		device_template_form_actions();

		break;
	case 'item_remove_gt':
		device_template_item_remove_gt();

		header("Location: device_templates.php?action=edit&tab=graph_templates&id=" . $_GET["device_template_id"]);
		break;
	case 'item_remove_dq':
		device_template_item_remove_dq();

		header("Location: device_templates.php?action=edit&tab=data_queries&id=" . $_GET["device_template_id"]);
		break;
	case 'edit':
		include_once(CACTI_BASE_PATH . "/include/top_header.php");

		device_template_edit();

		include_once(CACTI_BASE_PATH . "/include/bottom_footer.php");
		break;
	default:
		include_once(CACTI_BASE_PATH . "/include/top_header.php");

		device_template();

		include_once(CACTI_BASE_PATH . "/include/bottom_footer.php");
		break;
}

/* --------------------------
    The Save Function
   -------------------------- */

function device_template_form_save() {

	if (isset($_POST["save_component_template"])) {
		/* ================= input validation ================= */
		input_validate_input_number(get_request_var_post("id"));
		/* ==================================================== */
		$redirect_back = false;
		$tab = "&tab=general";

		$save["id"] 					= get_request_var_post("id");
		$save["hash"]					= get_hash_device_template($_POST["id"]);
		$save["name"]					= form_input_validate($_POST["name"], "name", "", false, 3);
		$save["description"]			= form_input_validate($_POST["description"], "description", "", true, 3);
		$save["image"]					= form_input_validate($_POST["image"], "image", "", true, 3);
		$save["override_defaults"]		= form_input_validate((isset($_POST["override_defaults"]) ? "on":""), "override_defaults", "", true, 3);
		$save["override_permitted"]		= form_input_validate((isset($_POST["override_permitted"]) ? "on":""), "override_permitted", "", true, 3);
		$save["snmp_version"]			= form_input_validate($_POST["snmp_version"], "snmp_version", "", true, 3);
		$save["snmp_version"]			= form_input_validate($_POST["snmp_version"], "snmp_version", "", true, 3);
		$save["snmp_community"]			= form_input_validate($_POST["snmp_community"], "snmp_community", "", true, 3);
		$save["snmp_username"]			= form_input_validate($_POST["snmp_username"], "snmp_username", "", true, 3);
		$save["snmp_password"]			= form_input_validate($_POST["snmp_password"], "snmp_password", "", true, 3);
		$save["snmp_auth_protocol"]		= form_input_validate($_POST["snmp_auth_protocol"], "snmp_auth_protocol", "", true, 3);
		$save["snmp_priv_passphrase"]	= form_input_validate($_POST["snmp_priv_passphrase"], "snmp_priv_passphrase", "", true, 3);
		$save["snmp_priv_protocol"]		= form_input_validate($_POST["snmp_priv_protocol"], "snmp_priv_protocol", "", true, 3);
		$save["snmp_context"]			= form_input_validate($_POST["snmp_context"], "snmp_context", "", true, 3);
		$save["snmp_port"]				= form_input_validate($_POST["snmp_port"], "snmp_port", "^[0-9]+$", false, 3);
		$save["snmp_timeout"]			= form_input_validate($_POST["snmp_timeout"], "snmp_timeout", "^[0-9]+$", false, 3);
		$save["availability_method"]	= form_input_validate($_POST["availability_method"], "availability_method", "^[0-9]+$", false, 3);
		$save["ping_method"]			= form_input_validate($_POST["ping_method"], "ping_method", "^[0-9]+$", false, 3);
		$save["ping_port"]				= form_input_validate($_POST["ping_port"], "ping_port", "^[0-9]+$", true, 3);
		$save["ping_timeout"]			= form_input_validate($_POST["ping_timeout"], "ping_timeout", "^[0-9]+$", true, 3);
		$save["ping_retries"]			= form_input_validate($_POST["ping_retries"], "ping_retries", "^[0-9]+$", true, 3);
		$save["max_oids"]				= form_input_validate($_POST["max_oids"], "max_oids", "^[0-9]+$", true, 3);
		$save["device_threads"]			= form_input_validate($_POST["device_threads"], "device_threads", "^[0-9]+$", true, 3);

		if (!is_error_message()) {
			$device_template_id = sql_save($save, "device_template");

			if ($device_template_id) {
				raise_message(1);
			}else{
				raise_message(2);
			}
		}
	}

	header("Location: device_templates.php?action=edit&tab=general&id=" . (empty($device_template_id) ? $_POST["id"] : $device_template_id));
	exit;
}

/**
 * Add a new Graph Template to the current Device Template.
 * Optionally, ask for confirmation to update related devices.
 */
function device_template_form_save_gt() {
	require_once(CACTI_BASE_PATH . "/lib/functions.php");

	/* ================= input validation ================= */
	input_validate_input_number(get_request_var_post("id"));
	input_validate_input_number(get_request_var_post("graph_template_id"));
	/* ==================================================== */

	/* We are on the graph templates tab.
	 * The user just requested to add a new graph template to this device template.
	 *
	 * We will add the graph template to the device template.
	 * Then, we will perform two runs:
	 * 1. We will prompt the user with a list of devices to be updated and ask for confirmation
	 *    If the device list is empty, there will be NO confirmation!
	 * 2. After confirmation. we will actually update the devices
	 */

	/* We will prompt the user with a list of devices to be updated and ask for confirmation */
	if (!isset($_POST["selected_items"])) {

		/*
		 * unconditionally add the new graph template to this device template
		 * this DOES NOT require a confirmation!
		 */
		db_execute("REPLACE INTO device_template_graph (device_template_id,graph_template_id) VALUES(" .
					get_request_var_post("id") . "," . get_request_var_post("graph_template_id") . ")");

		/*
		 * list all related hosts for confirmation,
		 * but omit those devices that already have this graph template
		 * */
		$new_gt_device_entries = db_fetch_assoc("SELECT device.id AS device_id, " .
											"device.description AS description, " .
											"device.hostname AS hostname " .
											"FROM device " .
											"WHERE	device.device_template_id = " . get_request_var_post("id") . " " .
											"AND		device.id NOT IN (" .
												"SELECT device_graph.device_id " .
												"FROM   device_graph " .
												"WHERE  device_graph.graph_template_id = " . get_request_var_post("graph_template_id") .
											")");

		/* if there are devices to work on */
		if (sizeof($new_gt_device_entries) > 0) {
			$device_list = ""; $i = 0; $device_array = array();
			/* fetch the graph template's name */
			$template_name = db_fetch_cell("SELECT name FROM graph_templates WHERE id = " . get_request_var_post("graph_template_id"));

			/* list all devices to be treated for confirmation */
			foreach($new_gt_device_entries as $entry) {
				$device_list .= "<li>" . $entry["hostname"] . " - " . $entry["description"] . "</li>\n";
				$device_array[$i++] = $entry["device_id"];
			}

			/* now draw the html page */
			include_once(CACTI_BASE_PATH . "/include/top_header.php");
			html_start_box("<strong>" . __("Confirm") . "</strong>", "60", "3", "center", "");

			print "<form action='device_templates.php' method='post' id='device_template_add_gt'>";
			# pass device template id and graph_template id to the updating code below
			form_hidden_box("id", get_request_var_post("id"), "");
			form_hidden_box("graph_template_id", get_request_var_post("graph_template_id"), "");
			print "
					<tr>
						<td class='textArea'>
							<p>" . __("Are you sure you want to add the following Graph Template:") . " <strong>" . $template_name . "</strong><br>" .
								__("All devices currently attached to the current Device Template will be updated.") . "</p>
							<p><ul>$device_list</ul></p>
						</td>
					</tr>\n
					";

			form_continue2(serialize($device_array), "save_gt");
			html_end_box();
			include_once(CACTI_BASE_PATH . "/include/bottom_footer.php");
			exit;
		}


	} else {
		/*
		 * 2. After confirmation. we will actually update the devices
		 */
		/* get all confirmed devices that were passed as a serializes array */
		$selected_items = unserialize(stripslashes($_POST["selected_items"]));
		if (sizeof($selected_items)) {

			foreach($selected_items as $device_id) {
				/* ================= input validation ================= */
				input_validate_input_number($device_id);
				/* ==================================================== */
				/* add the Graph Template */
				db_execute("REPLACE INTO device_graph ( device_id, graph_template_id ) VALUES (" .
							$device_id . "," . get_request_var_post("graph_template_id") . ")");
			}
		}
	}

	/* now let's return to the graph_templates tab of the current device template */
	header("Location: device_templates.php?action=edit&tab=graph_templates&id=" . get_request_var_post("id"));
	exit;

}


function device_template_form_save_dq() {
	/* required for "run_data_query" */
	include_once(CACTI_BASE_PATH . "/lib/data_query.php");

	/* We are on the data queries tab.
	 *
	 * User may request EITHER a reindex method update
	 *                  OR     adding a new data query
	 */

	if (isset($_POST["reindex"])) {
		/*
		 * loop for all possible changes of reindex_method
		 * post variable is build like this
		 * 		reindex_method_device_template_<device_id>_query_<snmp_query_id>_method_<old_reindex_method>
		 * if values of this variable differs from <old_reindex_method>, we will have to update
		 */
		while (list($var,$val) = each($_POST)) {
			if (preg_match("/^reindex_method_device_template_([0-9]+)_query_([0-9]+)_method_([0-9]+)$/", $var, $matches)) {
				/* ================= input validation ================= */
				input_validate_input_number(get_request_var_post("id"));
				input_validate_input_number($matches[1]); # device_template
				input_validate_input_number($matches[2]); # snmp_query_id
				input_validate_input_number($matches[3]); # old reindex method
				$reindex_method = $val;
				input_validate_input_number($reindex_method); # new reindex_method
				/* ==================================================== */

				# change reindex method of this very item
				if ( $reindex_method != $matches[3]) {
					db_execute("REPLACE INTO device_template_snmp_query (device_template_id,snmp_query_id,reindex_method) VALUES (" .
								$matches[1] . "," . $matches[2] . "," . $reindex_method .
								")");
					$reindex_performed = true;
				}
			}
		}


	}elseif (!isset($_POST["selected_items"])) {
		/*
		 * The user just requested to add a new data query to this device template.
		 *
		 * We will add the data query to the device template.
		 * Then, we will perform two runs:
		 * 1. We will prompt the user with a list of devices to be updated and ask for confirmation
		 *    If the device list is empty, there will be NO confirmation!
		 * 2. After confirmation. we will actually update the devices
		 */

		/* We will prompt the user with a list of devices to be updated and ask for confirmation */

		/* ================= input validation ================= */
		input_validate_input_number(get_request_var_post("id"));
		input_validate_input_number(get_request_var_post("snmp_query_id"));
		input_validate_input_number(get_request_var_post("reindex_method"));
		/* ==================================================== */

		/*
		 * unconditionally add the new data query to this device template
		 * this DOES NOT require a confirmation!
		 */
		db_execute("REPLACE INTO device_template_snmp_query (device_template_id,snmp_query_id, reindex_method) VALUES(" .
					get_request_var_post("id") . "," . get_request_var_post("snmp_query_id") . ", " . get_request_var_post("reindex_method") . ")");

		/*
		 * list all related hosts for confirmation,
		 * but omit those devices that already have this graph template
		 * */
		$new_dq_device_entries = db_fetch_assoc("SELECT device.id AS device_id, " .
											"device.description AS description, " .
											"device.hostname AS hostname " .
											"FROM  	device " .
											"WHERE	device.device_template_id = " . get_request_var_post("id") . " " .
											"AND	device.id NOT IN (" .
												"SELECT device_snmp_query.device_id " .
												"FROM   device_snmp_query " .
												"WHERE  device_snmp_query.snmp_query_id = " . $_POST["snmp_query_id"] .
											")");


		if (sizeof($new_dq_device_entries) > 0) {
			$device_list = ""; $i = 0; $device_array = array();
			/* fetch the graph template's name */
			$template_name = db_fetch_cell("SELECT name FROM snmp_query WHERE id = " . get_request_var_post("snmp_query_id"));

			/* list all devices to be treated for confirmation */
			foreach($new_dq_device_entries as $entry) {
				$device_list .= "<li>" . $entry["hostname"] . " - " . $entry["description"] . "</li>\n";
				$device_array[$i++] = $entry["device_id"];
			}

			/* now draw the html page */
			include_once(CACTI_BASE_PATH . "/include/top_header.php");
			html_start_box("<strong>" . __("Confirm") . "</strong>", "60", "3", "center", "");


			print "<form action='device_templates.php' method='post' id='device_template_add_dq'>";
			# pass device template id, data query id and reindex method to the updating code below
			form_hidden_box("id", get_request_var_post("id"), "");
			form_hidden_box("snmp_query_id", get_request_var_post("snmp_query_id"), "");
			form_hidden_box("reindex_method", get_request_var_post("reindex_method"), "");
			print "
					<tr>
						<td class='textArea'>
							<p>" . __("Are you sure you want to add the following Data Query:") . " <strong>" . $template_name . "</strong><br>" .
								__("All devices currently attached to the current Device Template will be updated.") . "</p>
							<p><ul>$device_list</ul></p>
						</td>
					</tr>\n
					";

			form_continue2(serialize($device_array), "save_dq");
			html_end_box();
			include_once(CACTI_BASE_PATH . "/include/bottom_footer.php");
			exit;
		}


	} else {
		/*
		 * 2. After confirmation. we will actually update the devices
		 */
		/* get all confirmed devices that were passed as a serializes array */
		$selected_items = unserialize(stripslashes($_POST["selected_items"]));
		if (sizeof($selected_items)) {

			foreach($selected_items as $device_id) {
				/* ================= input validation ================= */
				input_validate_input_number($device_id);
				/* ==================================================== */
				/* add the Data Query */
				db_execute("REPLACE INTO device_snmp_query (device_id,snmp_query_id,reindex_method)
							VALUES (". $device_id . ","
							. get_request_var_post("snmp_query_id") . ","
							. get_request_var_post("reindex_method") . "
							)");

				/* recache snmp data */
				run_data_query($device_id, get_request_var_post("snmp_query_id"));
			}
		}
	}


	header("Location: device_templates.php?action=edit&tab=data_queries&id=" . (empty($device_template_id) ? $_POST["id"] : $device_template_id));
	exit;
}


/* ------------------------
    The "actions" function
   ------------------------ */

function device_template_form_actions() {
	global $device_actions;

	/* if we are to save this form, instead of display it */
	if (isset($_POST["selected_items"])) {
		$selected_items = unserialize(stripslashes($_POST["selected_items"]));

		if (get_request_var_post("drp_action") === "1") { /* delete */
			/* do a referential integrity check */
			if (sizeof($selected_items)) {
			foreach($selected_items as $template_id) {
				/* ================= input validation ================= */
				input_validate_input_number($template_id);
				/* ==================================================== */

				if (sizeof(db_fetch_assoc("SELECT * FROM device WHERE device_template_id=$template_id LIMIT 1"))) {
					$bad_ids[] = $template_id;
				}else{
					$template_ids[] = $template_id;
				}
			}
			}

			if (isset($bad_ids)) {
				$message = "";
				foreach($bad_ids as $template_id) {
					$message .= (strlen($message) ? "<br>":"") . "<i>Device Template " . $template_id . " is in use and can not be removed</i>\n";
				}

				$_SESSION['sess_message_dt_ref_int'] = array('message' => "<font size=-2>$message</font>", 'type' => 'info');

				raise_message('dt_ref_int');
			}

			if (isset($template_ids)) {
				db_execute("delete from device_template where " . array_to_sql_or($template_ids, "id"));
				db_execute("delete from device_template_snmp_query where " . array_to_sql_or($template_ids, "device_template_id"));
				db_execute("delete from device_template_graph where " . array_to_sql_or($template_ids, "device_template_id"));

				/* "undo" any device that is currently using this template */
				db_execute("update device set device_template_id=0 where " . array_to_sql_or($template_ids, "device_template_id"));
			}
		}elseif (get_request_var_post("drp_action") === "2") { /* duplicate */
			for ($i=0;($i<count($selected_items));$i++) {
				/* ================= input validation ================= */
				input_validate_input_number($selected_items[$i]);
				/* ==================================================== */

				duplicate_device_template($selected_items[$i], get_request_var_post("title_format"));
			}
		}

		header("Location: device_templates.php");
		exit;
	}

	/* setup some variables */
	$device_list = ""; $device_array = array();

	/* loop through each of the device templates selected on the previous page and get more info about them */
	while (list($var,$val) = each($_POST)) {
		if (preg_match("/^chk_([0-9]+)$/", $var, $matches)) {
			/* ================= input validation ================= */
			input_validate_input_number($matches[1]);
			/* ==================================================== */

			$device_list .= "<li>" . db_fetch_cell("select name from device_template where id=" . $matches[1]) . "</li>";
			$device_array[] = $matches[1];
		}
	}

	include_once(CACTI_BASE_PATH . "/include/top_header.php");

	html_start_box("<strong>" . $device_actions{get_request_var_post("drp_action")} . "</strong>", "60", "3", "center", "");

	print "<form action='device_templates.php' method='post'>\n";

	if (sizeof($device_array)) {
		if (get_request_var_post("drp_action") === ACTION_NONE) { /* NONE */
			print "	<tr>
						<td class='textArea'>
							<p>" . __("You did not select a valid action. Please select 'Return' to return to the previous menu.") . "</p>
						</td>
					</tr>\n";
		}elseif (get_request_var_post("drp_action") === "1") { /* delete */
			print "	<tr>
					<td class='textArea'>
						<p>" . __("When you click 'Continue', the following Device Template(s) will be deleted.  All devices currently attached this these Device Template(s) will lose their template assocation.") . "</p>
						<p><ul>$device_list</ul></p>
					</td>
				</tr>\n";

			$title = __("Delete Device Template(s)");
		}elseif (get_request_var_post("drp_action") === "2") { /* duplicate */
			print "	<tr>
					<td class='textArea'>
						<p>" . __("When you click 'Continue', the following Device Template(s) will be duplicated. You can optionally change the title format for the new Device Template(s).") . "</p>
						<p><ul>$device_list</ul></p>
						<p><strong>" . __("Title Format:") . "</strong><br>"; form_text_box("title_format", "<template_title> (1)", "", "255", "30", "text"); print "</p>
					</td>
				</tr>\n";

			$title = __("Duplicate Device Template(s)");
		}
	} else {
		print "	<tr>
				<td class='textArea'>
					<p>" . __("You must first select a Device Template.  Please select 'Return' to return to the previous menu.") . "</p>
				</td>
			</tr>\n";
	}

	if (!sizeof($device_array) || get_request_var_post("drp_action") === ACTION_NONE) {
		form_return_button();
	}else{
		form_continue(serialize($device_array), get_request_var_post("drp_action"), $title);
	}

	html_end_box();

	include_once(CACTI_BASE_PATH . "/include/bottom_footer.php");
}

/* ---------------------
    Template Functions
   --------------------- */

function device_template_item_remove_gt() {
	/* ================= input validation ================= */
	input_validate_input_number(get_request_var("id"));
	input_validate_input_number(get_request_var("device_template_id"));
	/* ==================================================== */

	db_execute("delete from device_template_graph where graph_template_id=" . $_GET["id"] . " and device_template_id=" . $_GET["device_template_id"]);
}

function device_template_item_remove_dq() {
	/* ================= input validation ================= */
	input_validate_input_number(get_request_var("id"));
	input_validate_input_number(get_request_var("device_template_id"));
	/* ==================================================== */

	db_execute("delete from device_template_snmp_query where snmp_query_id=" . $_GET["id"] . " and device_template_id=" . $_GET["device_template_id"]);
}

function device_template_edit() {
	/* ================= input validation ================= */
	input_validate_input_number(get_request_var("id"));
	/* ==================================================== */

	/* remember if there's something we want to show to the user */
	$device_template_tabs = array(
		"general" 			=> __("General"),
		"graph_templates" 	=> __("Associated Graph Templates"),
		"data_queries" 		=> __("Associated Data Queries"),
		"devices" 			=> __("Devices"),
	);

	if (!empty($_REQUEST["id"])) {
		$device_template = db_fetch_row("select * from device_template where id=" . $_REQUEST["id"]);
		$header_label = __("[edit: ") . $device_template["name"] . "]";
		if (!db_fetch_cell("SELECT COUNT(*) FROM device WHERE device_template_id=" . $_REQUEST["id"])) {
			unset($device_template_tabs["devices"]);
		}
	}else{
		$device_template = array();
		$header_label = __("[new]");
		$_REQUEST["id"] = 0;
		unset($device_template_tabs["devices"]);
	}

	/* set the default settings category */
	if (!isset($_REQUEST["tab"])) {
		/* there is no selected tab; select the first one */
		$current_tab = array_keys($device_template_tabs);
		$current_tab = $current_tab[0];
	}else{
		$current_tab = $_REQUEST["tab"];
	}

	/* draw the categories tabs on the top of the page */
	print "<table width='100%' cellspacing='0' cellpadding='0' align='center'><tr>";
	print "<td><div class='tabs'>";

	if (sizeof($device_template_tabs) > 0) {
		foreach (array_keys($device_template_tabs) as $tab_short_name) {
			print "<div class='tabDefault'><a " . (($tab_short_name == $current_tab) ? "class='tabSelected'" : "class='tabDefault'") . " href='" . htmlspecialchars("device_templates.php?action=edit" . (isset($_REQUEST['id']) ? "&id=" . $_REQUEST['id'] . "&template_id=" . $_REQUEST['id']: "") . "&tab=$tab_short_name") . "'>$device_template_tabs[$tab_short_name]</a></div>";

			if (!isset($_REQUEST["id"])) break;
		}
	}
	print "</div></td></tr></table>";

	if (!isset($_REQUEST["tab"])) {
		$_REQUEST["tab"] = "general";
	}

	switch (get_request_var_request("tab")) {
		case "devices":
			include_once(CACTI_BASE_PATH . "/lib/device/device_form.php");
			include_once(CACTI_BASE_PATH . "/lib/utility.php");
			include_once(CACTI_BASE_PATH . "/lib/api_data_source.php");
			include_once(CACTI_BASE_PATH . "/lib/device/device_form.php");
			include_once(CACTI_BASE_PATH . "/lib/graph/graphs_new_form.php");
			include_once(CACTI_BASE_PATH . "/lib/graph/graphs_form.php");
			include_once(CACTI_BASE_PATH . "/lib/data_source/data_source_form.php");
			include_once(CACTI_BASE_PATH . "/lib/api_tree.php");
			include_once(CACTI_BASE_PATH . "/lib/html_tree.php");
			include_once(CACTI_BASE_PATH . "/lib/api_graph.php");
			include_once(CACTI_BASE_PATH . "/lib/data_query.php");
			include_once(CACTI_BASE_PATH . "/lib/sort.php");
			include_once(CACTI_BASE_PATH . "/lib/html_form_template.php");
			include_once(CACTI_BASE_PATH . "/lib/template.php");
			include_once(CACTI_BASE_PATH . "/lib/snmp.php");
			include_once(CACTI_BASE_PATH . "/lib/ping.php");
			include_once(CACTI_BASE_PATH . "/lib/api_device.php");

			device();

			break;

		case "graph_templates":
			if (!empty($_REQUEST["id"])) {
				device_template_display_gt($device_template, $header_label);
			}

			break;


		case "data_queries":
			if (!empty($_REQUEST["id"])) {
				device_template_display_dq($device_template, $header_label);
			}

			break;

		default:
			device_template_display_general($device_template, $header_label);

			break;
	}
}


function device_template_display_general($device_template, $header_label) {
	require(CACTI_BASE_PATH . "/include/data_query/data_query_arrays.php");
	require_once(CACTI_BASE_PATH . "/lib/device_template/device_template_info.php");

	print "<form method='post' action='" .  basename($_SERVER["PHP_SELF"]) . "' name='device_template_edit'>\n";
	html_start_box("<strong>" . __("Device Templates") . "</strong> $header_label", "100", "0", "center", "", true);
	$header_items = array(
		array("name" => __("Field")),
		array("name" => __("Value"))
	);

	print "<tr><td>";
	html_header($header_items, 1, false, 'device_template');

	draw_edit_form(array(
		"config" => array("no_form_tag" => true),
		"fields" => inject_form_variables(device_template_form_list(), (isset($device_template) ? $device_template : array()))
		));

	print "</table></td></tr>";		/* end of html_header */
	html_end_box();
	form_hidden_box("id", (isset($device_template["id"]) ? $device_template["id"] : "0"), "");
	form_hidden_box("save_component_template", "1", "");

	?>
	<script type="text/javascript">
	<!--

	// default snmp information
	var snmp_community       = document.getElementById('snmp_community').value;
	var snmp_username        = document.getElementById('snmp_username').value;
	var snmp_password        = document.getElementById('snmp_password').value;
	var snmp_auth_protocol   = document.getElementById('snmp_auth_protocol').value;
	var snmp_priv_passphrase = document.getElementById('snmp_priv_passphrase').value;
	var snmp_priv_protocol   = document.getElementById('snmp_priv_protocol').value;
	var snmp_context         = document.getElementById('snmp_context').value;
	var snmp_port            = document.getElementById('snmp_port').value;
	var snmp_timeout         = document.getElementById('snmp_timeout').value;
	var max_oids             = document.getElementById('max_oids').value;
	var device_threads       = document.getElementById('device_threads').value;

	// default ping methods
	var ping_method    = document.getElementById('ping_method').value;
	var ping_port      = document.getElementById('ping_port').value;
	var ping_timeout   = document.getElementById('ping_timeout').value;
	var ping_retries   = document.getElementById('ping_retries').value;

	// default availability methods
	var availability_methods = document.getElementById('availability_method').options;
	var num_methods          = document.getElementById('availability_method').length;
	var selectedIndex        = document.getElementById('availability_method').selectedIndex;

	var agent = navigator.userAgent;
	agent = agent.match("MSIE");

	function setPingVisibility() {
		availability_method = document.getElementById('availability_method').value;
		ping_method         = document.getElementById('ping_method').value;

		/* debugging, uncomment as required */
		//alert("The availability method is '" + availability_method + "'");
		//alert("The ping method is '" + ping_method + "'");

		switch(availability_method) {
		case "0": // none
			document.getElementById('row_ping_method').style.display  = "none";
			document.getElementById('row_ping_port').style.display    = "none";
			document.getElementById('row_ping_timeout').style.display = "none";
			document.getElementById('row_ping_retries').style.display = "none";

			break;
		case "2": // snmp
			document.getElementById('row_ping_method').style.display  = "none";
			document.getElementById('row_ping_port').style.display    = "none";
			document.getElementById('row_ping_timeout').style.display = "";
			document.getElementById('row_ping_retries').style.display = "";

			break;
		default: // ping ok
			switch(ping_method) {
			case "1": // ping icmp
				document.getElementById('row_ping_method').style.display  = "";
				document.getElementById('row_ping_port').style.display    = "none";
				document.getElementById('row_ping_timeout').style.display = "";
				document.getElementById('row_ping_retries').style.display = "";

				break;
			case "2": // ping udp
			case "3": // ping tcp
				document.getElementById('row_ping_method').style.display  = "";
				document.getElementById('row_ping_port').style.display    = "";
				document.getElementById('row_ping_timeout').style.display = "";
				document.getElementById('row_ping_retries').style.display = "";

				break;
			}

			break;
		}
	}

	function addSelectItem(item, formObj) {
		if (agent != "MSIE") {
			formObj.add(item,null); // standards compliant
		}else{
			formObj.add(item);      // IE only
		}
	}

	function setAvailability(type) {
		/* get the availability structure */
		var am=document.getElementById('availability_method');

		/* get current selectedIndex */
		selectedIndex = document.getElementById('availability_method').selectedIndex;

		/* debugging uncomment as required */
		//alert("The selectedIndex is '" + selectedIndex + "'");
		//alert("The array length is '" + am.length + "'");

		switch(type) {
		case "NoSNMP":
			/* remove snmp options */
			if (am.length == 4) {
				am.remove(1);
				am.remove(1);
			}

			/* set the index to something valid, like "ping" */
			if (selectedIndex > 1) {
				am.selectedIndex=1;
			}

			break;
		case "All":
			/* restore all options */
			if (am.length == 2) {
				am.remove(0);
				am.remove(0);

				var a=document.createElement('option');
				var b=document.createElement('option');
				var c=document.createElement('option');
				var d=document.createElement('option');

				a.value="0";
				a.text="<?php print __("None");?>";
				addSelectItem(a,am);

				b.value="1";
				b.text="<?php print __("Ping and SNMP");?>";
				addSelectItem(b,am);

				c.value="2";
				c.text="<?php print __("SNMP");?>";
				addSelectItem(c,am);

				d.value="3";
				d.text="<?php print __("Ping");?>";
				addSelectItem(d,am);

				/* restore the correct index number */
				if (selectedIndex == 0) {
					am.selectedIndex = 0;
				}else{
					am.selectedIndex = 3;
				}
			}

			break;
		}

		setAvailabilityVisibility(type, am.selectedIndex);
		setPingVisibility();
	}

	function setAvailabilityVisibility(type, selectedIndex) {
		switch(type) {
		case "NoSNMP":
			switch(selectedIndex) {
			case "0": // availability none
				document.getElementById('row_ping_method').style.display="none";
				document.getElementById('ping_method').value=0;

				break;
			case "1": // ping
				document.getElementById('row_ping_method').style.display="";
				document.getElementById('ping_method').value=ping_method;

				break;
			}
		case "All":
			switch(selectedIndex) {
			case "0": // availability none
				document.getElementById('row_ping_method').style.display="none";
				document.getElementById('ping_method').value=0;

				break;
			case "1": // ping and snmp
			case "3": // ping
				if ((document.getElementById('row_ping_method').style.display == "none") ||
					(document.getElementById('row_ping_method').style.display == undefined)) {
					document.getElementById('ping_method').value=ping_method;
					document.getElementById('row_ping_method').style.display="";
				}

				break;
			case "2": // snmp
				document.getElementById('row_ping_method').style.display="none";
				document.getElementById('ping_method').value="0";

				break;
			}
		}
	}

	function changeHostForm() {
		snmp_version        = document.getElementById('snmp_version').value;

		switch(snmp_version) {
		case "0":
			setAvailability("NoSNMP");
			setSNMP("None");

			break;
		case "1":
		case "2":
			setAvailability("All");
			setSNMP("v1v2");

			break;
		case "3":
			setAvailability("All");
			setSNMP("v3");

			break;
		}
	}

	function setSNMP(snmp_type) {
		switch(snmp_type) {
		case "None":
			document.getElementById('row_snmp_username').style.display        = "none";
			document.getElementById('row_snmp_password').style.display        = "none";
			document.getElementById('row_snmp_community').style.display       = "none";
			document.getElementById('row_snmp_auth_protocol').style.display   = "none";
			document.getElementById('row_snmp_priv_passphrase').style.display = "none";
			document.getElementById('row_snmp_priv_protocol').style.display   = "none";
			document.getElementById('row_snmp_context').style.display         = "none";
			document.getElementById('row_snmp_port').style.display            = "none";
			document.getElementById('row_snmp_timeout').style.display         = "none";
			document.getElementById('row_max_oids').style.display             = "none";

			break;
		case "v1v2":
			document.getElementById('row_snmp_username').style.display        = "none";
			document.getElementById('row_snmp_password').style.display        = "none";
			document.getElementById('row_snmp_community').style.display       = "";
			document.getElementById('row_snmp_auth_protocol').style.display   = "none";
			document.getElementById('row_snmp_priv_passphrase').style.display = "none";
			document.getElementById('row_snmp_priv_protocol').style.display   = "none";
			document.getElementById('row_snmp_context').style.display         = "none";
			document.getElementById('row_snmp_port').style.display            = "";
			document.getElementById('row_snmp_timeout').style.display         = "";
			document.getElementById('row_max_oids').style.display             = "";

			break;
		case "v3":
			document.getElementById('row_snmp_username').style.display        = "";
			document.getElementById('row_snmp_password').style.display        = "";
			document.getElementById('row_snmp_community').style.display       = "none";
			document.getElementById('row_snmp_auth_protocol').style.display   = "";
			document.getElementById('row_snmp_priv_passphrase').style.display = "";
			document.getElementById('row_snmp_priv_protocol').style.display   = "";
			document.getElementById('row_snmp_context').style.display         = "";
			document.getElementById('row_snmp_port').style.display            = "";
			document.getElementById('row_snmp_timeout').style.display         = "";
			document.getElementById('row_max_oids').style.display             = "";

			break;
		}
	}

	function toggleAvailabilityAndSnmp(show) {
		if (show) {
			$('#row_override_permitted').show();
			$('#row_availability_header').show();
			$('#row_availability_method').show();
			$('#row_ping_method').show();
			$('#row_ping_port').show();
			$('#row_ping_timeout').show();
			$('#row_ping_retries').show();
			$('#row_snmp_spacer').show();
			$('#row_snmp_version').show();
			$('#row_snmp_username').show();
			$('#row_snmp_password').show();
			$('#row_snmp_community').show();
			$('#row_snmp_auth_protocol').show();
			$('#row_snmp_priv_passphrase').show();
			$('#row_snmp_priv_protocol').show();
			$('#row_snmp_context').show();
			$('#row_snmp_port').show();
			$('#row_snmp_timeout').show();
			$('#row_max_oids').show();
			$('#row_device_threads').show();

			changeHostForm();
		}else{
			$('#row_override_permitted').hide();
			$('#row_availability_header').hide();
			$('#row_availability_method').hide();
			$('#row_ping_method').hide();
			$('#row_ping_port').hide();
			$('#row_ping_timeout').hide();
			$('#row_ping_retries').hide();
			$('#row_snmp_spacer').hide();
			$('#row_snmp_version').hide();
			$('#row_snmp_username').hide();
			$('#row_snmp_password').hide();
			$('#row_snmp_community').hide();
			$('#row_snmp_auth_protocol').hide();
			$('#row_snmp_priv_passphrase').hide();
			$('#row_snmp_priv_protocol').hide();
			$('#row_snmp_context').hide();
			$('#row_snmp_port').hide();
			$('#row_snmp_timeout').hide()
			$('#row_max_oids').hide();
			$('#row_device_threads').hide();
		}
	}

	/* jQuery stuff */
	$().ready(function() {
		toggleAvailabilityAndSnmp(document.getElementById('override_defaults').checked);

		/* Hide options when override is turned off */
		$("#override_defaults").change(function () {
			toggleAvailabilityAndSnmp(this.checked);
		});

		/* Hide "Uptime Goes Backwards" if snmp_version has been set to "None" */
		$("#snmp_version").change(function () {
				/* get PHP constants into javascript namespace */
				var reindex_none = <?php print DATA_QUERY_AUTOINDEX_NONE;?>;
				var reindex_reboot = <?php print DATA_QUERY_AUTOINDEX_BACKWARDS_UPTIME;?>;
				/* we require numeric values for comparison */
				var current_reindex = parseInt($(this).val());
				switch (current_reindex)
				{
					case reindex_none:
						/* now that SNMP is disabled, select reindex method "None" */
						$("#reindex_method option[value=" + reindex_none + "]").attr('selected', 'true');
						/* disable SNMP options: "Uptime Goes Backwards" never works with pure Script Data Queries */
						$("#reindex_method option[value=" + reindex_reboot + "]").attr('disabled', 'true');
						$("#reindex_method option[value=" + reindex_reboot + "]").attr('title', '<?php print __("Disabled due to SNMP settings");?>');
						break;
					default:
						/* "Uptime Goes Backwards" is allowed again */
						$("#reindex_method option[value=" + reindex_reboot + "]").removeAttr("disabled");
						$("#reindex_method option[value=" + reindex_reboot + "]").attr('title', '');
						/* select this again as default reindex method */
						/* TODO: this ignores the default reindex method of the associated Device Template
						   to get it, an AJAX call is required */
						$("#reindex_method option[value=" + reindex_reboot + "]").attr('selected', 'true');
			}
		});
	});

	//-->
	</script>
	<?php

	form_save_button("device_templates.php", "return");
}


function device_template_display_gt($device_template, $header_label) {
	print "<form method='post' action='" .  basename($_SERVER["PHP_SELF"]) . "' name='device_template_gt_edit'>\n";
	html_start_box("<strong>" . __("Associated Graph Templates") . " $header_label</strong>", "100", "3", "center", "", true);
	print "<tr><td>";
	html_header(array(array("name" => __("Graph Template Name"))), 3);

	/* list all graph templates that are currentyl associated */
	$selected_graph_templates = db_fetch_assoc("SELECT " .
			"graph_templates.id, " .
			"graph_templates.name " .
			"FROM device_template_graph " .
			"LEFT JOIN graph_templates ON (device_template_graph.graph_template_id = graph_templates.id) " .
			"WHERE device_template_graph.device_template_id=" . $device_template["id"] . " " .
			"ORDER BY graph_templates.name");

	/* Now list all graph templates, that have NOT yet been associated.
	 * This is to prevent duplicate assignments for the same graph template
	 */
	$available_graph_templates = db_fetch_assoc("SELECT " .
			"graph_templates.id, " .
			"graph_templates.name " .
			"FROM graph_templates " .
			"WHERE graph_templates.id NOT IN (" .
				"SELECT graph_templates.id " .
				"FROM device_template_graph " .
				"LEFT JOIN graph_templates ON (device_template_graph.graph_template_id = graph_templates.id) " .
				"WHERE device_template_graph.device_template_id=" . $device_template["id"] .
			") " .
			"ORDER BY graph_templates.name");


	$i = 0;
	if (sizeof($selected_graph_templates) > 0) {
		foreach ($selected_graph_templates as $item) {
			form_alternate_row_color("selected_graph_template" . $item["id"], true);
			$i++;
			?>
				<td style="padding: 4px;"><strong><?php print $i;?>)</strong> <?php print $item["name"];?>
				</td>
				<td align='right' nowrap><input
					type='button'
					value='<?php print __("Remove");?>'
					onClick='document.location="<?php print htmlspecialchars("device_templates.php?action=item_remove_gt&id=" . $item["id"] . "&device_template_id=" . $device_template["id"]);?>"'
					title='<?php print __("Delete Graph Template Association");?>'>
				</td>
			<?php
			form_end_row();
		}
	}else{
		print "<tr><td><em>" . __("No associated graph templates.") . "</em></td></tr>";
	}

	form_alternate_row_color("add_template" . get_request_var("id"), false);
	?>
	<td nowrap><?php form_dropdown("graph_template_id",$available_graph_templates,"name","id","","","");?>
	</td>
	<td align="right">&nbsp;<input type="submit"
		value="<?php print __("Add Template");?>" name="add_gt_y">
	</td>
	<input type='hidden' name='action' value='save_gt'>
	<?php
	form_end_row();
	print "</table></td></tr>";		/* end of html_header */
	html_end_box(true);
	form_hidden_box("id", (isset($device_template["id"]) ? $device_template["id"] : "0"), "");
	print "</form>";
}


function device_template_display_dq($device_template, $header_label) {
	require(CACTI_BASE_PATH . "/include/data_query/data_query_arrays.php");

	print "<form method='post' action='" .  basename($_SERVER["PHP_SELF"]) . "' name='device_template_dq_edit'>\n";
	html_start_box("<strong>" . __("Associated Data Queries") . " $header_label</strong>", "100", "0", "center", "", true);
	print "<tr><td>";
	html_header(array(array("name" => __("Data Query Name")), array("name" => __("Re-Index Method"))), 2);

	/* list all data queries that are currently associated */
	$selected_data_queries = db_fetch_assoc("SELECT " .
			"snmp_query.id, " .
			"snmp_query.name, " .
			"device_template_snmp_query.reindex_method " .
			"FROM device_template_snmp_query " .
			"LEFT JOIN snmp_query ON (device_template_snmp_query.snmp_query_id = snmp_query.id) " .
			"WHERE device_template_snmp_query.device_template_id=" . $device_template["id"] . " " .
			"ORDER BY snmp_query.name");

	/* Now list all data queries, that have NOT yet been associated.
	 * This is to prevent duplicate assignments for the same data query
	 */
	$available_data_queries = db_fetch_assoc("SELECT " .
			"snmp_query.id, " .
			"snmp_query.name " .
			"FROM snmp_query " .
			"WHERE snmp_query.id NOT IN (" .
				"SELECT snmp_query.id " .
				"FROM device_template_snmp_query " .
				"LEFT JOIN snmp_query ON (device_template_snmp_query.snmp_query_id = snmp_query.id) " .
				"WHERE device_template_snmp_query.device_template_id=" . $device_template["id"] .
			") " .
			"ORDER BY snmp_query.name");

	$i = 0;
	if (sizeof($selected_data_queries) > 0) {
		foreach ($selected_data_queries as $item) {
			form_alternate_row_color("selected_data_query" . $item["id"], true);
			$i++;
			?>
				<td style="padding: 4px;"><strong><?php print $i;?>)</strong>
					<?php print $item["name"];?>
				</td>
				<td><?php form_dropdown("reindex_method_device_template_".get_request_var("id")."_query_".$item["id"]."_method_".$item["reindex_method"],$reindex_types,"","",$item["reindex_method"],"","","","");?>
				</td>
				<td align='right' nowrap><input
					type='button'
					value='<?php print __("Remove");?>'
					onClick='document.location="<?php print htmlspecialchars("device_templates.php?action=item_remove_dq&id=" . $item["id"] . "&device_template_id=" . $device_template["id"]);?>'
					title='<?php print __("Delete Graph Template Association");?>'>
				</td>
			<?php
			form_end_row();
		}
	}else{
		print "<tr><td><em>" . __("No associated data queries.") . "</em></td></tr>";
	}

	/* add new data queries */
	form_alternate_row_color("add_data_query", false);
	?>
	<td nowrap><?php form_dropdown("snmp_query_id",$available_data_queries,"name","id","","","");?>
	</td>
	<td nowrap><?php form_dropdown("reindex_method",$reindex_types,"","","1","","");?>
	</td>
	<td align="right">&nbsp;<input type="submit"
		value="<?php print __("Add Data Query");?>" name="add_dq_y" align="middle">
	</td>
	<?php
	form_end_row();

	/* update the reindex methods */
	form_alternate_row_color("reindex", false);
	?>
	<td nowrap colspan="3" align="right">&nbsp;<input type="submit"
		value="<?php print __("Update Re-Index Methods");?>" name="reindex">
	</td>
	<?php
	form_end_row();

	print "</table></td></tr>";		/* end of html_header */
	html_end_box(true);
	form_hidden_box("action", "save_dq", "save_dq");
	form_hidden_box("id", (isset($device_template["id"]) ? $device_template["id"] : "0"), "");
	print "</form>";
}

function device_templates_filter() {
	global $item_rows;

	html_start_box("<strong>" . __("Device Templates") . "</strong>", "100", "3", "center", "device_templates.php?action=edit", true);
	?>
	<tr class='rowAlternate2'>
		<td>
			<form name="form_device_template" action="device_templates.php">
			<table cellpadding="0" cellspacing="3">
				<tr>
					<td class="nw50">
						&nbsp;<?php print __("Search:");?>&nbsp;
					</td>
					<td width="1">
						<input type="text" name="filter" size="40" value="<?php print html_get_page_variable("filter");?>">
					</td>
					<td class="nw50">
						&nbsp;<?php print __("Rows:");?>&nbsp;
					</td>
					<td width="1">
						<select name="rows" onChange="applyFilterChange(document.form_device_template)">
							<option value="-1"<?php if (html_get_page_variable("rows") == "-1") {?> selected<?php }?>>Default</option>
							<?php
							if (sizeof($item_rows) > 0) {
							foreach ($item_rows as $key => $value) {
								print "<option value='" . $key . "'"; if (html_get_page_variable("rows") == $key) { print " selected"; } print ">" . $value . "</option>\n";
							}
							}
							?>
						</select>
					</td>
					<td class="nw120">
						&nbsp;<input type="submit" Value="<?php print __("Go");?>" name="go" align="middle">
						<input type="submit" Value="<?php print __("Clear");?>" name="clear" align="middle">
					</td>
				</tr>
			</table>
			<div><input type='hidden' name='page' value='1'></div>
			</form>
		</td>
	</tr>
	<?php
	html_end_box(false);
	?>
	<script type="text/javascript">
	<!--
	function applyFilterChange(objForm) {
		strURL = '?rows=' + objForm.rows.value;
		strURL = strURL + '&filter=' + objForm.filter.value;
		document.location = strURL;
	}
	-->
	</script>
	<?php
}

function get_device_template_records(&$total_rows, &$rowspp) {
	/* form the 'where' clause for our main sql query */
	if (strlen(html_get_page_variable("filter"))) {
		$sql_where = "WHERE (device_template.name LIKE '%%" . html_get_page_variable("filter") . "%%')
			OR (device_template.description LIKE '%%" . html_get_page_variable("filter") . "%%')";
	}else{
		$sql_where = "";
	}

	if (html_get_page_variable("rows") == "-1") {
		$rowspp = read_config_option("num_rows_device");
	}else{
		$rowspp = html_get_page_variable("rows");
	}

	$total_rows = db_fetch_cell("SELECT
		COUNT(device_template.id)
		FROM device_template
		$sql_where");

	return db_fetch_assoc("SELECT *
		FROM device_template
		$sql_where
		ORDER BY " . html_get_page_variable('sort_column') . " " . html_get_page_variable('sort_direction') .
		" LIMIT " . ($rowspp*(html_get_page_variable("page")-1)) . "," . $rowspp);
}

function device_template($refresh = true) {
	global $device_actions;

	$table = New html_table;

	$table->page_variables = array(
		"page"           => array("type" => "numeric", "method" => "request", "default" => "1"),
		"rows"           => array("type" => "numeric", "method" => "request", "default" => "-1"),
		"filter"         => array("type" => "string",  "method" => "request", "default" => ""),
		"sort_column"    => array("type" => "string",  "method" => "request", "default" => "name"),
		"sort_direction" => array("type" => "string",  "method" => "request", "default" => "ASC")
	);

	$table->table_format = array(
		"name" => array(
			"name" => __("Template Title"),
			"filter" => true,
			"link" => true,
			"order" => "ASC"
		),
		"description" => array(
			"name" => __("Description"),
			"filter" => true,
			"link" => true,
			"order" => "ASC"
		),
		"id" => array(
			"name" => __("ID"),
			"order" => "ASC"
		),
		"nosort" => array(
			"name" => __("Availbility/SNMP Settings"),
			"function" => "display_device_template_control",
			"params" => array("override_defaults", "override_permitted"),
			"sort" => false
		),
		"image" => array(
			"name" => __("Image"),
			"sort" => false,
			"align" => "center"
		)
	);

	/* initialize page behavior */
	$table->href           = "device_templates.php";
	$table->session_prefix = "sess_device_templates";
	$table->filter_func    = "device_templates_filter";
	$table->refresh        = $refresh;
	$table->resizable      = true;
	$table->checkbox       = true;
	$table->sortable       = true;
	$table->actions        = $device_actions;

	/* we must validate table variables */
	$table->process_page_variables();

	/* get the records */
	$table->rows = get_device_template_records($table->total_rows, $table->rows_per_page);

	/* display the table */
	$table->draw_table();
}

function display_device_template_control($override_defaults, $override_permitted) {
	return ($override_defaults == "on" ? __("Template controls Availability and SNMP") . ($override_permitted == "on" ? __(", User can override"):__(", Template propagation is forced")):__("Using System Defaults"));
}
